/* Wrap Gladman's code
 */
 
#ifndef __ED_C_SHA_H
#define __ED_C_SHA_H

#include "pasis_types.h"
#include "pasis_debug.h"


//See the following for SHA spec:
//http://csrc.nist.gov/encryption/shs/sha256-384-512.pdf


//! implements the EDCSHACryptoPP-256 standard
class ED_C_SHA256_G
{
public:
    static const unsigned int DIGEST_SIZE = 32;
    static const unsigned int BLOCK_SIZE = 64;
    static const unsigned int LENGTH_MARKER_SIZE = 8;

    ED_C_SHA256_G() {}

    //digests are not ?transitive?
    //digest([0 1 2 3], 4, &d1 );
    //digest([4 5 6 7], 4, &d2 );
    //digest([0 1 2 3 4 5 6 7], 8, &d );
    //d != d1 ^ d2; 
    //This is because this digest function appends virtual padding and
    //a virtual byte count to meet the SHA spec.
    static void digest( const p_byte *buffer,
                        unsigned int length, 
                        p_byte *digest );

    static void digest( const p_byte **buffers,
                        unsigned int *buffer_lengths, 
                        unsigned int num_buffers,
                        p_byte *digest );

    static void key_digest( const p_byte *key,
                            unsigned int key_length,
                            const p_byte *buffer,
                            unsigned int buffer_length, 
                            p_byte *digest );
                         
    static void key_digest( const p_byte *key,
                            unsigned int key_length,
                            const p_byte **buffers,
                            unsigned int *buffer_lengths, 
                            unsigned int num_buffers,
                            p_byte *digest );
                         
 private:

    static const p_uint32 DIGEST_INIT[8];
    static const p_uint32 K[64];
};



#endif
