/* cred_cache.c
   cgeisser@andrew.cmu.edu
   based on code by rjs3@andrew.cmu.edu
   
*/


#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <strings.h>
#include <stdlib.h>
#include "cred_cache.h"

#define HT_BUCKETS 251


/* The hash table */
static struct hash_entry *ht[HT_BUCKETS];

/* For the timeout list */
static struct hash_entry *ht_q = NULL;

static u_int hash_val(char *key1, u_int key2, u_int key3) {
    u_int total = 0;
    u_int i = 0;

    for (i=0; key1[i]; i++) {
        total ^= key1[i] << ((i%4)*4);
    }

    total ^= key2 ^ key3;

    total = total % HT_BUCKETS;

    //printf("hash code %d\n", total);
    return total;
} 

void hash_item(char* princ, u_int cli, u_int server, time_t exptime,
	       u_int keysize, byte *key) 
{
    u_int hashcode;
    struct hash_entry *temp;
    struct hash_entry *itr, *itr_next, **prv;

    hashcode = hash_val(princ, cli, server);
    temp = malloc(sizeof(struct hash_entry));
    bzero(temp, sizeof(struct hash_entry));

    if(!temp) {
        fprintf(stderr, "Out of Memory!\n");
        exit(1);
    }

    temp->my_code = hashcode;
    temp->timestamp = exptime;
    temp->princ = strdup(princ);
    temp->client = cli;
    temp->server = server;
    temp->keysize = keysize;
    if (key && keysize > 0) {
        //printf("new key\n");
        temp->key = (byte*) malloc(keysize*sizeof(byte));
        memcpy(temp->key, key, keysize*sizeof(byte));
    }


    /* Insert onto timeout queue */
    if(!ht_q) {
        ht_q = temp;
        //printf("q first\n");
        temp->next_q = temp->prev_q = NULL;
    } else if (ht_q->timestamp > temp->timestamp) {
        temp->next_q = ht_q;
        ht_q->prev_q = temp;
        ht_q = temp;
        
    } else {
        temp->next_q = NULL;
        temp->prev_q = NULL;
        for (itr = ht_q; itr && itr->next_q &&
                 itr->next_q->timestamp < temp->timestamp;
             itr=itr->next_q);
        temp->next_q = itr->next_q;
        itr->next_q = temp;
        temp->prev_q = itr;
        if (temp->next_q)
            temp->next_q->prev_q = temp;
    }

    if (ht[hashcode] == NULL) {
        ht[hashcode] = temp;
        temp->next = NULL;
        temp->prev = NULL;
    } else {
        //printf("doing stuff\n");
        for(itr = ht[hashcode], prv = &(ht[hashcode]);
            itr; itr=itr_next) {
            itr_next = itr->next;

            if(!strcmp(princ, itr->princ) &&
               server == itr->server && 
               cli == itr->client) {
                /* Duplicate! Remove it! */
                //printf("dup!\n");

                /* Fix the hash table */
                *prv = itr->next;

                /* Fix the timeout list */
                if(itr->prev_q) 
                    itr->prev_q->next_q = itr->next_q;
                else 
                    ht_q = itr->next_q;

                if(itr->next_q) 
                    itr->next_q->prev_q = itr->prev_q;
	

                /* Free it */
                free(itr->princ);
                free(itr->key);
                free(itr);
            } else {
                prv = &(itr->next);
            }
        }

        /* Insert current item into hash table (in front so it will show
         * up at the front of the list in get requests) */
        temp->next = ht[hashcode];
        temp->prev = NULL;
        if(temp->next) 
            temp->next->prev = temp;
        ht[hashcode] = temp;
    }
}
const int hash_nextexp_time(void) {
    if (ht_q)
        return ht_q->timestamp;
    else
        return -1;

}
  
struct hash_entry* hash_nextexp(void) 
{
    struct hash_entry *t = NULL;
    
    if (ht_q) {
        t = ht_q;

        /* Remove from timeout queue */
        ht_q = t->next_q;
        if(ht_q) 
            ht_q->prev_q = NULL;
    
        /* Remove from hash table */
        if (t->prev)
            t->prev->next = t->next;
        else
            ht[t->my_code] = t->next;

        if (t->next) 
            t->next->prev = t->prev;

    }

    return t;
}

struct hash_entry *hash_get(char* princ, u_int client, u_int server) 
{

    struct hash_entry *temp;
    u_int hashcode;
    
    hashcode = hash_val(princ, client, server);
    if (ht[hashcode] == NULL) {
        //printf("no hashed value\n");
        return NULL;
    } else {
        for(temp=ht[hashcode]; temp; temp=temp->next) {
            if(server == temp->server && 
               !strcmp(princ, temp->princ) &&               
               client == temp->client) {
                /* match return */
                return temp;
            }
        }
    }

    return NULL;
}

void hash_init(void)
{
    ht_q = NULL;
    memset(ht, 0, sizeof(ht));
}

void hash_shutdown(void) {
    struct hash_entry *itr = ht_q;
    
    for (itr = ht_q; itr; itr = itr->next_q) {
        if (itr->prev_q) {
            free(itr->prev_q->princ);
            free(itr->prev_q->key);
            free(itr->prev_q);
        }
        
    }
}

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
