#include "krb_box.h"
#include "cred_cache.h"
#include "../tools/state_threads/include/st.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <arpa/inet.h>

#include <krb5.h>
//#include <com_err.h>


static krb5_principal my_princ;
static krb5_context context;
static st_thread_t recv_thread;
static krb5_error_code retval;
//static char* progname = "krb keycache";
static int running = 0;
static char my_hostname[MAXHOSTNAMELEN+1];
static u_int my_addr;
static struct sockaddr_in s_sock;		/* server address */
static st_netfd_t st_outsock;
static int outsock;

void* recv_handler(void* arg) 
{
    unsigned char pktbuf[INPUT_BUF_SIZE];
    struct sockaddr_in c_sock;		/* client's address */
    krb5_data ap_rep, packet;
    krb5_auth_context auth_context = NULL;

    krb5_ticket *ticket = NULL;
    struct hostent *host = NULL;
    krb5_keyblock         *session_keyblock = NULL;
    int i = 0;
    int sz = sizeof(struct sockaddr_in);
    char* cp = NULL;


    printf("recieve thread listening\n");
    while (running) {
        if ((i = st_recvfrom(st_outsock, (char *)pktbuf, sizeof(pktbuf), 
                            (struct sockaddr *)&c_sock, (int*) &sz , -1)) < 0) {
            if (errno == EINTR)
                return 0;
            else
                perror("receiving datagram");

        }

        if (i>0) {

            printf("Received %d bytes from %d\n", i, c_sock.sin_addr.s_addr );
            packet.length = i;
            packet.data = (krb5_pointer) pktbuf;
            
            /* Check authentication info */
            if ((retval = krb5_rd_req(context, &auth_context, &packet, 
                                      my_princ, NULL, NULL, &ticket)) != 0) {
                krb5_err(context, 1, retval, "while reading request");
                exit(1);
            }
            if ((retval = krb5_unparse_name(context, ticket->client,
                                            &cp)) != 0) {
                krb5_err(context, 1, retval, "while unparsing client name");
                exit(1);
            }
            printf("Got authentication info from %s\n", cp);
            krb5_auth_con_getkey(context, auth_context, &session_keyblock);

            krb5_mk_rep(context, auth_context, &ap_rep);

            if (st_sendto(st_outsock, ap_rep.data, ap_rep.length, (struct sockaddr *)&c_sock, 
                          sizeof(struct sockaddr_in), NET_TO) < 0) {
                if (errno == EINTR)
                    return 0;
                else
                    perror("error while sending");
            }
                

            printf("sent reply\n");
            
            host = gethostbyaddr((void*)&c_sock.sin_addr, sizeof(c_sock.sin_addr), AF_INET);

            hash_item(cp, c_sock.sin_addr.s_addr, my_addr, ticket->ticket.endtime,
                      session_keyblock->keyvalue.length,
                      session_keyblock->keyvalue.data);

            //printf("%x\n", (uint) packet.data);
            //printf("%x\n", (uint) &ap_rep);

            krb5_free_data_contents(context, &ap_rep);
            /*krb5_free_data_contents(context, &packet);*/
            krb5_free_ticket(context, ticket);

            krb5_auth_con_free(context, auth_context);
            krb5_free_keyblock(context, session_keyblock);
            free(cp);
            auth_context = NULL;
            session_keyblock = NULL;
            ticket = NULL;
            cp = NULL;
        }
    }

    return NULL;

    
}


int krb_init_keycache_srv() 
{

    char* service = SERVICE_NAME;
    struct hostent *host;
    
    if (running) {
        printf("already initialized!\n");
        return -1;
    }

    retval = krb5_init_context(&context);
    printf("initializing kerberos\n");
    if (retval) {
        krb5_err(context, 1, retval, "while initializing krb5");
        exit(1);
    }
        
    hash_init();
    //st_init();
    
    gethostname(my_hostname, MAXHOSTNAMELEN);
    
    /* get my principal */
    if ((retval = krb5_sname_to_principal(context, NULL, service, 
					  KRB5_NT_SRV_HST, &my_princ))) {
	krb5_err(context, 1, retval, "while generating service name %s", service);
	exit(1);
    }
    
    (void) memset((char *)&s_sock, 0, sizeof(s_sock));
    if ((host = gethostbyname(my_hostname)) == (struct hostent *)0) {
	fprintf(stderr, "%s: host unknown\n", my_hostname);
	exit(1);
    }
    memcpy((char *)&s_sock.sin_addr, host->h_addr, sizeof(s_sock.sin_addr));
    my_addr = s_sock.sin_addr.s_addr;
    printf("I am %d\n", my_addr);
    s_sock.sin_family = AF_INET;
    s_sock.sin_port = htons(AUTH_PORT);
    if ((outsock = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
	perror("opening datagram socket");
	exit(1);
    }
    /* Let the socket be reused right away */
    (void) setsockopt(outsock, SOL_SOCKET, SO_REUSEADDR, (char*)1,
                      sizeof(char*));
    
    st_outsock = st_netfd_open(outsock);
    /* Bind the socket */
    
    if (bind(outsock, (struct sockaddr *)&s_sock, sizeof(s_sock))) {
	perror("binding datagram socket");
	exit(1);
    }
    running = 1;
    recv_thread = st_thread_create(recv_handler, NULL, 1, 1<<18);

    return 0;
    
}



octet_string* krb_get_sh_key_srv(char* princ, u_int client, u_int server) {

    struct hash_entry* hash_found = NULL;
    octet_string* ret_key = NULL;

    if (!running) {
        printf("not initialized!\n");
        return NULL;
    }
    
    printf("trying to get key for %s %d %d\n", princ, client, server);

    hash_found = hash_get(princ, client, server);
    //hash_found = NULL; 
    
    if (hash_found && hash_found->keysize != 0) {
        printf("found\n");
        ret_key = (octet_string*) malloc(sizeof(octet_string));
        ret_key->length = hash_found->keysize;
        ret_key->data = (byte*) malloc(ret_key->length*sizeof(byte));
        memcpy(ret_key->data, hash_found->key, ret_key->length);
    } else {
        return NULL;
    }
    
    return ret_key;

}

int krb_shutdown_srv(void) {
    //do something
    if (!running) {
        printf("not initialized!\n");
        return -1;
    }
    running = 0;
    st_thread_interrupt(recv_thread);
    st_thread_join(recv_thread, NULL);
    st_netfd_close(st_outsock);
    hash_shutdown();
    
    krb5_free_principal(context, my_princ);
    krb5_free_context(context);

    return 0;
}


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
