/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISFS_NFS_FE_H__
#define __PASISFS_NFS_FE_H__

#include "mount_prot.h"
#include "nfs_prot.h"

extern 
void PasisFS_GetNFSuid(const struct svc_req *rq, unsigned *uid, unsigned *gid);

nfsstat PasisFS_NFSGetRoot (nfs_fh *fh_out);

nfsstat PasisFS_NFSGetAttr (unsigned int uid, nfs_fh *fh, fattr *fa_out);

nfsstat PasisFS_NFSSetAttr (unsigned int uid, nfs_fh *fh, sattr *sa_in, 
			    fattr *fa_out);

nfsstat PasisFS_NFSLookup (unsigned int uid, nfs_fh *dir, filename file, 
			   nfs_fh *file_out, fattr *fa_out);

nfsstat PasisFS_NFSReadlink (unsigned int uid, nfs_fh *fh, char *path);

nfsstat PasisFS_NFSLink (unsigned int uid, nfs_fh *from_fh, nfs_fh *to_dir, 
			 filename to_name);

nfsstat PasisFS_NFSSymlink (unsigned int uid, nfs_fh *from_dir, 
			    filename from_name, nfspath link, sattr *sattr);

nfsstat PasisFS_NFSRead (unsigned int uid, nfs_fh *fh, char *buffer, 
			 unsigned int offset, unsigned int len, 
			 unsigned int *len_out, fattr *fa_out);

nfsstat PasisFS_NFSWrite (unsigned int uid, nfs_fh *fh, char *buffer, 
			  unsigned int offset, unsigned int len, 
			  unsigned int *len_out, fattr *fa_out);

nfsstat PasisFS_NFSCreate (unsigned int uid, nfs_fh *dir, filename name, 
			   sattr *sa_in, nfs_fh *file_out, fattr *fa_out);

nfsstat PasisFS_NFSUnlink (unsigned int uid, nfs_fh *dir, filename name);

nfsstat PasisFS_NFSRename (unsigned int uid, nfs_fh *from_dir, 
			   filename from_name,
			   nfs_fh *to_dir, filename to_name);

nfsstat PasisFS_NFSMkdir (unsigned int uid, nfs_fh *dir, filename name, 
			  sattr *sa_in, nfs_fh *file_out, fattr *fa_out);

nfsstat PasisFS_NFSRmdir (unsigned int uid, nfs_fh *dir, filename name);

nfsstat PasisFS_NFSReaddir (unsigned int uid, nfs_fh *dir, int count, 
			    p_uint32 cookie, struct dirlist *dirlist);

nfsstat PasisFS_NFSStat (nfs_fh *fh, statfsokres *stat);

#endif /* __PASISFS_NFS_FE_H__ */
