/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#include "pasis_config.h"
#include "pasis_debug.h"
#include "pasis_error.h"

#include "mount1_prot.h"
#include "nfs_prot.h"
#include "nfs3_prot.h"

#include "fe_rpc.h"
#include "fe_xdr.h"
#include "xdr_mount1.h"

#include "nfs3_fe.h"

#define RPC_IN_ARGS(rpc_state) \
  ((u_int32_t *)((rpc_state)->in_iov[1].iov_base))

#define RPC_OUT_ARGS(rpc_state) \
  ((u_int32_t *)((rpc_state)->out_iov[1].iov_base))

#define RPC_SET_OUT_LEN(rpc_state, p) \
  (rpc_state)->out_iov[1].iov_len = ((char *)p - (char *)rpc_state->out_iov[1].iov_base);

#define RPC_ZERO_OUT_LEN(rpc_state) ((rpc_state)->out_iov[1].iov_len = 0)

static bool_t
mountproc1_null (rpc_state_t *rpc_state)
{
    RPC_ZERO_OUT_LEN(rpc_state);
    return PASIS_TRUE;
}

static bool_t
mountproc1_mnt(rpc_state_t *rpc_state)
{
    dirpath   args;
    fhstatus  result;
    nfs_fh3   fh3;
    u_int32_t *p;

    int path_len;

    fh3.data.data_val = result.fhstatus_u.fhs_fhandle;
    fh3.data.data_len = FHSIZE;

    /* Decode args, dirpath set to point inside input rpc buffer */
    xdr_decode_dirpath (RPC_IN_ARGS (rpc_state), &args);

    PASIS_DPRINTF (PASIS_DEBUG_FS_NFS, 3, "%s: path=[%s]\n", __FUNCTION__, 
                   args);

    if (((path_len = strlen (args)) > MNTPATHLEN) || (args[0] != '/')) {
        result.fhs_status = NFSERR_INVAL;
        goto encode_mnt_result;
    }

    /* Remove trailing / */
    if (args[path_len-1] == '/') {
        args[path_len-1] = '\0';
    }

    result.fhs_status = PasisFS_NFS3GetRoot (rpc_state->rpc_auth_uid, args,
                                             (nfs_fh3 *)&fh3);

 encode_mnt_result:    
    p = xdr_encode_fhstatus (RPC_OUT_ARGS (rpc_state), &result); 
    RPC_SET_OUT_LEN (rpc_state, p);

    return PASIS_TRUE;
}

static bool_t
mountproc1_dump(rpc_state_t *rpc_state)
{
    /* XXX need to fill in result */
    //mountbody3 result;

    RPC_ZERO_OUT_LEN (rpc_state);

    return PASIS_TRUE;
}

static bool_t
mountproc1_umnt(rpc_state_t *rpc_state)
{
    //dirpath3 arg;
    RPC_ZERO_OUT_LEN (rpc_state);
    return PASIS_TRUE;
}

static bool_t
mountproc1_umntall(rpc_state_t *rpc_state)
{
    RPC_ZERO_OUT_LEN (rpc_state);
    return PASIS_TRUE;
}

static bool_t
mountproc1_export(rpc_state_t *rpc_state)
{
    /* XXX need to fill in result */
    //exports3 result;
    RPC_ZERO_OUT_LEN (rpc_state);
    return PASIS_TRUE;
}

static bool_t
mountproc1_exportall(rpc_state_t *rpc_state)
{
    RPC_ZERO_OUT_LEN (rpc_state);
    return PASIS_TRUE;
}

void 
mount_program_1(rpc_state_t *rpc_state)
{
    switch (rpc_state->rpc_proc) {
    case MOUNTPROC_NULL:
        mountproc1_null (rpc_state);
        break;
    case MOUNTPROC_MNT:
        mountproc1_mnt (rpc_state);
        break;
    case MOUNTPROC_DUMP:
        mountproc1_dump (rpc_state);
        break;
    case MOUNTPROC_UMNT:
        mountproc1_umnt (rpc_state);
        break;
    case MOUNTPROC_UMNTALL:
        mountproc1_umntall (rpc_state);
        break;
    case MOUNTPROC_EXPORT:
        mountproc1_export (rpc_state);
        break;
    case MOUNTPROC_EXPORTALL:
        mountproc1_exportall (rpc_state);
        break;
    }
}


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
