/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __FE_XDR_H__
#define __FE_XDR_H__

#define XDR_QUADLEN(l)          (((l) + 3) >> 2)
     
/* Common decode functions */
static inline u_int32_t *
xdr_decode_string (u_int32_t *p, char **str)
{
  unsigned int    len;
  char            *string;
  
  len = ntohl (*p++);

  if ((len % 4) != 0) {
    string = (char *) p;
  } else {
    string = (char *) (p - 1);
    memmove(string, p, len);
  }

  string[len] = '\0';
  *str = string;

  return p + XDR_QUADLEN (len);
}

static inline u_int32_t *
xdr_decode_hyper (u_int32_t *p, u_int64_t *hyper)
{
    *hyper = ((u_int64_t) ntohl (*p++)) << 32;
    *hyper |= ntohl (*p++);

    return p;
}

/* Common encoding functions */ 
static inline u_int32_t *
xdr_encode_string (u_int32_t *p, char *string)
{
    unsigned int len;
    len = strlen (string);
    
    *p++ = htonl (len);
    memcpy (p, string, len);

    return p + XDR_QUADLEN (len); 
}

static inline u_int32_t *
xdr_encode_hyper (u_int32_t *p, u_int64_t *hyper)
{
    *p++ = htonl(*hyper >> 32);
    *p++ = htonl(*hyper & 0xFFFFFFFF);
    return p;
}


#endif /* __FE_XDR_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
