/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASISIO_MD_PROTOCOL_H__
#define __PASISIO_MD_PROTOCOL_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasisio.h"
#include "pasis_md_prot.h"
#include "pasisio_md_hist_cache.h"


#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif



typedef struct pasisio_md_proto_data_s {
    pasisio_md_hist_data_t  *fid_hist_cache[MD_MAX_OBJS_PER_REQ];

    /* Before sending a write request, we need to "backup" the authenticators
     * we're about to send.
     * To see why, consider client A sending a write request to a lot of S-N's.
     * As the first S-N's respond, they overwrite the authenticators for the
     * "current" request with their new authenticators.  This is bad, because
     * the client is still using the "old" history matrix (since that's already
     * been copied to the proto->write_op_args.
     * The solution is for the client to backup the authenticators before
     * sending a write request.  Then, when it is sending requests, it uses the
     * backed-up authenticators.  When it receives replies, it stores them in the
     * "current" authenticators set, which is in the fid_hist_cache above. */
    pasis_md_auth_t          auth_to_send[MD_MAX_OBJS_PER_REQ]
                                         [PASIS_MAX_NUM_SERVERS][PASIS_MAX_NUM_SERVERS];
    unsigned int             auth_to_send_num[MD_MAX_OBJS_PER_REQ]
                                             [PASIS_MAX_NUM_SERVERS];


    pasis_md_protocol_t     *protocol;
    pasis_md_linkage_t      *linkages;

    pasis_md_hts_t          *ts_classify;

    pasis_md_read_flag_t     read_flags[PASIS_MAX_NUM_SERVERS];

    pasis_md_read_op_args_t  read_op_args;
    pasis_md_read_op_res_t   read_op_res[PASIS_MAX_NUM_SERVERS];

    pasis_md_write_op_args_t write_op_args;
    pasis_md_write_op_res_t  write_op_res[PASIS_MAX_NUM_SERVERS];

    /* During repair, we need to backup the arguments/results.  This is because
     * the repair operatoin (reading operation/history, writing
     * barriers/operation) will overwrite the proto structure's
     * arguments/results. */
    unsigned int             already_saved_write_op; /* for sanity checking */
    pasis_md_read_op_args_specific_t  read_op_args_backup;
    pasis_md_write_op_args_specific_t write_op_args_backup;
    pasis_md_read_op_res_specific_t   read_op_res_backup[PASIS_MAX_NUM_SERVERS];
    pasis_md_write_op_res_specific_t  write_op_res_backup[PASIS_MAX_NUM_SERVERS];
    /* During barrier writes, we need to know the chunk size for synthetic
     * operations.  it's easiest to just save it here */
    p_uint32 chunk_size;

    unsigned int             shuffle;

    pasis_md_auth_t         *auths[MD_MAX_OBJS_PER_REQ]
                                  [PASIS_MAX_NUM_SERVERS]
                                  [PASIS_MAX_NUM_SERVERS];
} pasisio_md_proto_data_t;


/* specifies whether we're stress testing the system or not.  if set to 1, then
 * the code will be more lenient and allow incomplete writes, and not assert. */
extern int STRESS_TESTING; /* default = 0 */

/* Specifies whether we should update a client's history cache after the client
 * executes an operation.  Usually, we want to do this, but during testing, we
 * sometimes want this off (e.g., to allow incomplete after a repairable) */
extern int DO_NOT_UPDATE_HISTORY;



/* "Complete" is a historic term, and it does NOT refer to a complete quorum
 * size.  Instead, it refers to how many S-Ns a client must send a request to.
 * Think of this as "num_issue".
 * "early_term" is really the quorum size. */
void
PasisIO_GetCompleteThreshold (pasis_md_protocol_t *protocol,
                              unsigned int        *complete,
                              unsigned int        *early_term);

unsigned int
PasisIO_GetRepairableThreshold (pasis_md_protocol_t *protocol);


pasis_error_t
PasisIO_MDInit (unsigned int num_outstanding);

void
PasisIO_MDShutdown (void);


pasis_error_t
PasisIO_MDCreate (pasis_md_protocol_t *protocol,
                  pasis_md_linkage_t  *linkages,
                  unsigned int         num_linkages,
                  pasis_md_fid_t      *parent_fid,
                  pasis_md_fid_t      *name_fid,
                  char                *name,
                  char                *symlink_name,
                  pasis_md_attr_t     *name_attr_in,
                  pasis_md_protocol_t *name_protocol,
                  pasis_md_linkage_t  *name_linkages,
                  unsigned int         num_name_linkages,
                  pasis_md_attr_t     *parent_attr_out,    /* Out args */
                  pasis_md_attr_t     *name_attr_out);     /* Out args */

pasis_error_t
PasisIO_MDLink (pasis_md_protocol_t *protocol,
                pasis_md_linkage_t  *linkages,
                unsigned int         num_linkages,
                pasis_md_fid_t      *from_fid,
                pasis_md_fid_t      *to_dir_fid,
                char                *to_name,
                pasis_md_attr_t     *from_attr_out,       /* Out args */
                pasis_md_attr_t     *to_dir_attr_out);    /* Out args */


pasis_error_t
PasisIO_MDUnlink (pasis_md_protocol_t *protocol,
                  pasis_md_linkage_t  *linkages,
                  unsigned int         num_linkages,
                  pasis_md_fid_t      *parent_fid,
                  pasis_md_fid_t      *from_fid,
                  char                *from_name,
                  pasis_md_attr_t     *parent_attr_out);  /* Out args */


pasis_error_t
PasisIO_MDSetattr (pasis_md_protocol_t *protocol,
                   pasis_md_linkage_t  *linkages,
                   unsigned int         num_linkages,
                   pasis_md_fid_t      *fid,
                   pasis_md_sattr_t    *sattr,
                   p_uint32             guard_ctime[2],
                   pasis_md_attr_t     *attr_out);        /* Out args */


pasis_error_t
PasisIO_MDRename (pasis_md_protocol_t   *protocol,
                  pasis_md_linkage_t    *linkages,
                  unsigned int           num_linkages,
                  pasis_md_fid_t        *from_dir,
                  pasis_md_fid_t        *to_dir,
                  pasis_md_fid_t        *from_fid,
                  pasis_md_fid_t        *to_fid,
                  char                  *from_name,
                  char                  *to_name,
                  int                    to_name_valid,
                  pasis_md_attr_t       *from_dir_attr_out, /* Out args */
                  pasis_md_attr_t       *to_dir_attr_out);  /* Out args */


pasis_error_t
PasisIO_MDGetattr (pasis_md_protocol_t *protocol,
                   pasis_md_linkage_t  *linkages,
                   unsigned int         num_linkages,
                   pasis_md_fid_t      *fid,
                   pasis_md_attr_t     *attr_out,         /* Out args */
                   pasis_md_loc_t      *distribution_out);/* Out args */


pasis_error_t
PasisIO_MDLookup (pasis_md_protocol_t *protocol,
                  pasis_md_linkage_t  *linkages,
                  unsigned int         num_linkages,
                  pasis_md_fid_t      *parent_fid,
                  char                *name,
                  pasis_md_fid_t      *fid_out,           /* Out args */
                  pasis_md_attr_t     *dir_attr_out,      /* Out args */
                  pasis_md_loc_t      *distribution_out); /* Out args */


pasis_error_t
PasisIO_MDReadlink (pasis_md_protocol_t *protocol,
                    pasis_md_linkage_t  *linkages,
                    unsigned int         num_linkages,
                    pasis_md_fid_t      *fid,
                    char                *path_out,        /* Out args */
                    pasis_md_attr_t     *fid_attr_out);   /* Out args */

pasis_error_t
PasisIO_MDReaddir (pasis_md_protocol_t *protocol,
                   pasis_md_linkage_t  *linkages,
                   unsigned int         num_linkages,
                   pasis_md_fid_t      *fid,
                   p_uint64             blkno,
                   p_byte              *buffer_out,
                   pasis_md_attr_t     *dir_attr_out);   /* Out args */


pasis_error_t
PasisIO_MDSynthQuery (pasis_md_protocol_t   *protocol,
                      pasis_md_fid_t       **fids,
                      p_uint32               num_fids,
                      pasis_md_linkage_t    *linkages,
                      unsigned int           num_linkages,
                      p_uint32               arg_size,
                      p_uint32               res_size,
                      p_uint32               chunk_size,
                      p_uint32              *read_chunks, /* bitmaps, 1/object*/
                      p_uint32               compute_time, /* In milliseconds */
                      unsigned int           use_cached_hist_flag,
                      p_uint32               server_override_bitmap,
                      /* Only pass data if modified_chunks has just one
                       * element, and chunk_size = arg_size */
                      p_byte                *data);


pasis_error_t
PasisIO_MDSynthUpdate (pasis_md_protocol_t *protocol,
                       pasis_md_fid_t     **fids,
                       p_uint32             num_fids,
                       pasis_md_linkage_t  *linkages,
                       unsigned int         num_linkages,
                       p_uint32             arg_size,
                       p_uint32             res_size,
                       p_uint32             chunk_size,
                       p_uint32            *modified_chunks, /* bitmaps, 1/obj*/
                       p_uint32             compute_time, /* In milliseconds */
                       /* For testing, useful for client to force server to
                        * increment logical TS by more than 1 */
                       p_uint32             ts_increment, /* Usually 1 */
                       unsigned int         use_cached_hist_flag,
                       p_uint32             server_override_bitmap,
                       /* Only pass data if modified_chunks has just one
                        * element, and chunk_size = arg_size */
                       p_byte              *data);



pasis_error_t
PasisIO_MDObjectSync (pasis_md_protocol_t   *protocol,
                      pasis_md_fid_t        *fid,
                      pasis_md_linkage_t    *linkages,  /* B+1 servers */
                      pasis_md_hts_t        *sync_ts,
                      p_uint32               block_size,
                      p_byte                *buffer,    /* result */
                      p_uint32               starting_block_num,
                      p_uint32               num_blocks_to_ret,
                      p_uint32              *num_total_blocks, /* Result */
                      p_uint32              *num_returned_blocks); /* Result */


/* Get statistics from servers about how many requests they're handling */
pasis_error_t
PasisIO_MDGetStats (pasis_md_protocol_t   *protocol,
                    pasis_md_linkage_t    *linkages,
                    unsigned int           num_linkages,
                    /* stats is a caller-allocated array of size num_linkages,
                     * where each element points to an array of integers which
                     * will be filled in with statistics from the corresponding
                     * server in 'linkages'. */
                    p_uint32             **stats,    /* result */
                    /* Each element of the above array's size is "element_size"
                     * integers. We ask the server to return up to
                     * element_size. */
                    p_uint32               element_size,
                    /* flags is a caller-allocated array of size num_linkages
                     * which will have position 'i' set to 1 if we returned
                     * stats for the 'i'th server in the linkages */
                    p_byte                *flags,
                    /* num_buckets and bucket_size will be set on return. If the
                     * servers don't all return the same num_buckets and
                     * bucket_size parameter, then this function returns an
                     * error.  Obviously, this is not Byzantine-FT but this
                     * functionality addresses performance, not robustness. */
                    p_uint32              *num_buckets, /* result */
                    p_uint32              *bucket_size); /* result */

/* Notes:
 * -This function obeys the quorum constraints in "protocol". So if you want to
 * ensure that you gather statistics from _all_ servers, make sure to set
 * protocol as follows: protocol->quorum_type = PASISIO_QUORUM_NONE,
 * protocol->N = number ov servers, and protocol->T = 0.
 * -The elements in "stats" are returned with the most recent bucket counts
 * first.
 */



/* For testing/benchmarking, it's useful for a client program to be able to
 * force random quorum usage, as opposed to preferred quorums. */
void
PasisIO_UseRandomQuorums (void);

void
PasisIO_UsePreferredQuorums (void);



#ifdef __cplusplus
}
#endif

#endif /* __PASISIO_MD_PROTOCOL_H__ */


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */ 
