/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#include "pasis_config.h"
#include "pasis_debug.h"
#include "pasis_freelist.h"
#include "pasis_error.h"

#include "pasisio.h"
#include "pasisio_internal.h"
#include "pasisio_md_protocol.h"
#include "pasisio_drives.h"
#include "pasisio_select.h"


#define ED_THREAD_NUM  (1)


void
PasisIO_Shutdown (void)
{
    PasisIO_DriveShutdown ();
    PasisIO_MDShutdown ();

    return;
}


pasis_error_t
PasisIO_Init (unsigned int outstanding,
              unsigned int block_size)
{
    static int already_called = 0;
    pasis_error_t error;

    if (already_called) {
        PASIS_PRINT_ERR("%s: don't call this function more than once!  Memory "
                        "leaks would occur\n", __FUNCTION__);
        exit(-1);
    } else {
        already_called = 1;
    }

    /* Add 1 since the Q/U do_get_stat_thread (see pasisio_select.c) continually
     * does a PASIS request in the background (to retrieve the servers' load). */
    ++outstanding;

    if ((error = PasisIO_DriveInit (outstanding))) {
        return error;
    }

    if ((error = PasisIO_MDInit (outstanding))) {
        return error;
    }

    return PASIS_SUCCESS;
}


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
