/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_ATTR_H__
#define __PASIS_MD_ATTR_H__

#include "pasis_config.h"
#include "pasis_types.h"

/* Attribute structure */
typedef struct pasis_md_attr_s {
    p_uint32 mode;             /* Mode and type of file */
    p_uint32 ref_cnt;          /* Reference count for hard links */

    p_uint32 uid;              /* Owner user id */
    p_uint32 gid;              /* Group id */
    p_uint32 fileid;           /* File ID for NFS */
    p_uint32 mtime[2];         /* Last modified time {secs, nsecs} */
    p_uint32 atime[2];         /* Last accessed time */
    p_uint32 ctime[2];         /* Creation time */
 
    p_uint64 blocks;           /* Number of data blocks used */
    p_uint64 size;
} pasis_md_attr_t;


/* Used to set attributes */
typedef struct pasis_md_sattr_s {
    p_uint32 set_mode;
    p_uint32 mode;
    
    p_uint32 set_uid;
    p_uint32 uid;
    
    p_uint32 set_gid;
    p_uint32 gid;

    p_uint32 set_size;
    p_uint64 size;

    p_uint32 set_blocks;
    p_uint64 blocks;

    p_uint32 set_atime;
    p_uint32 atime[2];

    p_uint32 set_mtime;
    p_uint32 mtime[2];
} pasis_md_sattr_t;


/* Attribute mode bits -- taken from NFSv2/v3 */
#define MD_ATTR_MODE_MASK 0770000
#define MD_ATTR_MODE_DIR  0040000
#define MD_ATTR_MODE_CHR  0020000
#define MD_ATTR_MODE_BLK  0060000
#define MD_ATTR_MODE_REG  0100000
#define MD_ATTR_MODE_LNK  0120000
#define MD_ATTR_MODE_SOCK 0140000
#define MD_ATTR_MODE_FIFO 0010000


#define MD_ATTR_IS_DIR(attr) \
  ((((attr)->mode) & MD_ATTR_MODE_MASK) == MD_ATTR_MODE_DIR)

#define MD_ATTR_IS_LNK(attr) \
  ((((attr)->mode) & MD_ATTR_MODE_MASK) == MD_ATTR_MODE_LNK)

#define MD_ATTR_IS_REG(attr) \
  ((((attr)->mode) & MD_ATTR_MODE_MASK) == MD_ATTR_MODE_REG)


static inline void
pasis_md_attr_set_time (p_uint32 times[2])
{
    struct timeval tv;
    gettimeofday (&tv, NULL);
    times[0] = tv.tv_sec;
    times[1] = tv.tv_usec;
}

static inline void
pasis_md_attr_copy_time (p_uint32 dest[2], p_uint32 src[2])
{
    dest[0] = src[0];
    dest[1] = src[1];
}

static inline void
pasis_md_attr_update_time (p_uint32 old[2], p_uint32 neww[2])
{
    if (neww[0] >= old[0]) {  /* Only update if we are moving time forward */
        old[0] = neww[0];
        old[1] = neww[1];
    } else {
        old[1]++;
        if (old[1] == 0) old[0]++;
    }
}


#endif

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
