/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_AUTH_H__
#define __PASIS_MD_AUTH_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_error.h"

#include "ed_c_md5.h"
#include "ed_c_sha.h"

#include "pasis_md_prot.h"


#ifdef __cplusplus
extern "C" {
#endif



typedef struct auth_key_s {
    p_uint32  local_node_id;
    p_uint32 remote_node_id;
} auth_key_t;


/* Authentication code (for generating/checking authenticators) */


void pasis_md_auth_stats_init (void);
void pasis_md_auth_stats_dump (int shutdown_flag);

pasis_error_t
pasis_md_auth_init (void);


void
pasis_md_auth_shutdown (void);


/* Generate a set of N authenticators over a single history vector */
void
pasis_md_auth_generate (p_uint32         local_node_id,
                        p_uint32         history_num,
                        pasis_md_hts_t  *histories,
                        p_uint32         auth_num,    /* # of S-N's */
                        pasis_md_auth_t *authenticators);


/* Verify a single authenticator for an array of pointers to history entries.
 * Returns PASIS_SUCCESS on success, PASIS_ERROR otherwise. */
pasis_error_t
pasis_md_auth_verify (p_uint32            local_node_id,
                      p_uint32            remote_node_id,
                      p_uint32            history_num,
                      pasis_md_hts_t    **histories,
                      pasis_md_auth_t    *authenticator);
    
/* Hash a buffer. Caller passes in an output buffer that will be filled by the
 * hash.  It must of size ED_HASH_DIGEST_SIZE bytes. */
void
pasis_md_auth_hash (u_int32_t                size,  /* buffer size in bytes */
                    p_byte                  *buffer,
                    p_byte                  *hash); /* Out arg */


/* Takes an array of _pointers_ to history entries and returns the UNkeyed
 * hash. */ 
void
pasis_md_auth_hash_indirect (p_uint32           history_num,
                             pasis_md_hts_t   **histories,
                             pasis_md_auth_t   *hash);


/* Hash a history matrix. Caller passes in an output buffer that will be filled
 * by the hash.  It must of size ED_HASH_DIGEST_SIZE bytes. */
void
pasis_md_auth_hash_matrix (pasis_md_hist_matrix_t  *matrix,
                           p_byte                  *hash); /* Out arg */




/* Takes a keyed hash of a hash.
 * For performance reasons, this function is here, so we can avoid the function
 * call over head. */
static inline void
pasis_md_auth_key_hash (auth_key_t      *key,
                        pasis_md_auth_t *hash,
                        pasis_md_auth_t *authenticator)
{
    /* PASIS_TIMER_START (); */
    if (DO_NOT_CALCULATE_AUTHENTICATORS) {
        /* memset (authenticator, 0, ED_HASH_DIGEST_SIZE); */
    } else {
        HASH_FUNCTION::key_digest((p_byte *) key, sizeof(auth_key_t),
                                  (p_byte *) hash,
                                  PASIS_HASH_SIZE,
                                  (p_byte *) authenticator);
    }
    /* PASIS_TIMER_STOP (stats.timers, PASIS_MD_TIMER_AUTH_KEY_HASH); */
}


#ifdef __cplusplus
}
#endif


#endif /* __PASIS_MD_AUTH_H__ */


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
