/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_OPS_H__
#define __PASIS_MD_OPS_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "pasis_error.h"
#include "pasis_md_prot.h"

/* Header file for filesystem implementation function.
 * These functions are called after the necessary conditional write validation
 * has been performed. */


class MDVersionedObject;
class MDVersion;


/* Covers: create, mkdir, mknod, symlink */
pasis_error_t
pasis_md_op_create (p_uint32             mtime[2],
                    MDVersionedObject   *parent,
                    MDVersionedObject   *create,
                    pasis_md_hts_t      *new_hts,
                    pasis_md_hts_t     **cond_hts,
                    char                *create_name,
                    pasis_md_attr_t     *create_attr,
                    char                *symlink_path,
                    pasis_md_attr_t     *create_attr_out,   /* Out args */
                    pasis_md_attr_t     *parent_attr_out);  /* Out args */

pasis_error_t
pasis_md_op_link (p_uint32             mtime[2],
                  MDVersionedObject   *from,
                  MDVersionedObject   *to_dir,
                  pasis_md_hts_t      *new_hts,
                  pasis_md_hts_t     **cond_hts,
                  char                *to_name,
                  pasis_md_attr_t     *from_attr_out,      /* Out args */
                  pasis_md_attr_t     *to_dir_attr_out);   /* Out args */


pasis_error_t
pasis_md_op_unlink (p_uint32             mtime[2],
                    MDVersionedObject   *parent_dir,
                    MDVersionedObject   *from_fid,
                    pasis_md_hts_t      *new_hts,
                    pasis_md_hts_t     **cond_hts,
                    char                *from_name,
                    pasis_md_attr_t     *parent_attr_out); /* Out args */

pasis_error_t
pasis_md_op_setattr (MDVersionedObject    *fid,
                     pasis_md_hts_t       *new_hts,
                     pasis_md_hts_t      **cond_hts,
                     pasis_md_sattr_t     *sattr,
                     p_uint32              guard_ctime[2],
                     pasis_md_attr_t      *attr_out);      /* Out args */

pasis_error_t
pasis_md_op_rename (p_uint32             mtime[2],
                    MDVersionedObject   *from_dir,
                    MDVersionedObject   *to_dir,
                    MDVersionedObject   *from_fid,
                    MDVersionedObject   *to_fid,
                    pasis_md_hts_t      *new_hts,
                    pasis_md_hts_t     **cond_hts,
                    char                *from_name,
                    char                *to_name,
                    pasis_md_attr_t     *from_dir_attr_out, /* Out args */
                    pasis_md_attr_t     *to_dir_attr_out);  /* Out args */

pasis_error_t
pasis_md_op_getattr (MDVersionedObject   *file,
                     MDVersion           *version,
                     pasis_md_attr_t     *attr_out);       /* Out args */


pasis_error_t
pasis_md_op_lookup (MDVersionedObject   *directory,
                    MDVersion           *version,
                    char                *name,
                    pasis_md_fid_t      *fid_out,          /* Out args */
                    pasis_md_attr_t     *dir_attr_out);    /* Out args */

pasis_error_t
pasis_md_op_readlink (MDVersionedObject   *fid,
                      MDVersion           *version,
                      unsigned int        *path_len_out,   /* Out args */
                      char                *path_out,       /* Out args */
                      pasis_md_attr_t     *fid_attr_out);  /* Out args */



/* Creates root directory */
pasis_error_t
pasis_md_op_createroot (MDVersionedObject *root_object);


#endif /* __PASIS_MD_OPS_H__ */


/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
