/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_WORK_H__
#define __PASIS_MD_WORK_H__

#include "pasis_config.h"
#include "pasis_error.h"
#include "pasis_types.h"

#include "pasis_md_s4.h"


#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif


/* If we're not using a backend (i.e., running in-memory experiments). */
extern p_uint32 not_using_backend;


/* Node ID. = ED_SCHEME_MAX_N if server is masquering as multiple servers (uses
 * incoming partition as its node ID). */
extern p_uint32 node_id;


pasis_error_t
pasis_md_work_init (pasis_md_loc_t *distribution,
                    p_uint64        block_cache_size,
                    p_uint64        history_cache_size);

void
pasis_md_work_shutdown (void);

void
pasis_md_work_stats_init (void);

void
pasis_md_work_stats_dump (int shutdown_flag);




void
pasis_md_work_read_history (pasis_fe_s4_ipc_t       *ipc_op,
                            pasis_md_read_op_args_t *args,
                            pasis_md_read_op_res_t  *res);


void
pasis_md_work_create (pasis_md_write_op_args_t *args,
                      u_int32_t                 op_size,
                      u_int32_t                *encoded_op,
                      pasis_md_write_op_res_t  *res);


void
pasis_md_work_link (pasis_md_write_op_args_t *args,
                    u_int32_t                 op_size,
                    u_int32_t                *encoded_op,
                    pasis_md_write_op_res_t  *res);


void
pasis_md_work_unlink (pasis_md_write_op_args_t *args,
                      u_int32_t                 op_size,
                      u_int32_t                *encoded_op,
                      pasis_md_write_op_res_t  *res);


void
pasis_md_work_getattr (pasis_md_read_op_args_t *args,
                       pasis_md_read_op_res_t  *res);


void
pasis_md_work_setattr (pasis_md_write_op_args_t *args,
                       u_int32_t                 op_size,
                       u_int32_t                *encoded_op,
                       pasis_md_write_op_res_t  *res);


void
pasis_md_work_rename (pasis_md_write_op_args_t *args,
                      u_int32_t                 op_size,
                      u_int32_t                *encoded_op,
                      pasis_md_write_op_res_t  *res);


void
pasis_md_work_lookup (pasis_md_read_op_args_t *args,
                      pasis_md_read_op_res_t  *res);


void
pasis_md_work_readlink (pasis_md_read_op_args_t *args,
                        pasis_md_read_op_res_t  *res);


void
pasis_md_work_readdir (pasis_md_read_op_args_t *args,
                       pasis_md_read_op_res_t  *res);

void
pasis_md_work_barrier (pasis_fe_s4_ipc_t        *ipc_op,
                       pasis_md_write_op_args_t *args,
                       pasis_md_write_op_res_t  *res);

void
pasis_md_work_synth_query (pasis_fe_s4_ipc_t       *ipc_op,
                           pasis_md_read_op_args_t *args,
                           pasis_md_read_op_res_t  *res);

void
pasis_md_work_synth_update (pasis_md_write_op_args_t *args,
                            u_int32_t                 op_size,
                            u_int32_t                *encoded_op,
                            pasis_md_write_op_res_t  *res);



#ifdef __cplusplus
}
#endif



#endif /*  __PASIS_MD_WORK_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
