/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#ifndef __PASIS_MD_XDR_H__
#define __PASIS_MD_XDR_H__

#include "pasis_config.h"
#include "pasis_types.h"
#include "rpc_common.h"

#include "pasis_md_prot.h"


#ifdef __cplusplus
extern "C" {
#endif
#if 0
}
#endif

#ifndef TRUE
#define TRUE 1
#endif

#define XDR_QUADLEN(l)            (((l) + 3) >> 2)


/* For client transmit stub -- send args */
int pasis_md_xdr_enc_read_op_args (rpc_req_t *req, u_int32_t *p, 
                                   pasis_md_read_op_args_t *args);

int pasis_md_xdr_enc_write_op_args (rpc_req_t *req, u_int32_t *p, 
                                    pasis_md_write_op_args_t *args);


/* For server recv stub -- recv args */
int pasis_md_xdr_dec_read_op_args (u_int32_t *p, 
                                   pasis_md_read_op_args_t *args);

int pasis_md_xdr_dec_write_op_args (u_int32_t *p, 
                                    pasis_md_write_op_args_t *args,
                                    u_int32_t *op_size,
                                    u_int32_t **op);


/* For server recv stub -- send args */
int pasis_md_xdr_enc_read_op_res (rpc_req_t *req, u_int32_t *p, 
                                  pasis_md_read_op_res_t *res);

int pasis_md_xdr_enc_write_op_res (rpc_req_t *req, u_int32_t *p, 
                                   pasis_md_write_op_res_t *res);


/* For client recv stub -- recv results */
int pasis_md_xdr_dec_read_op_res (u_int32_t *p, 
                                  pasis_md_read_op_res_t *res);

int pasis_md_xdr_dec_write_op_res (u_int32_t *p, 
                                   pasis_md_write_op_res_t *res);



/* This is useful for clients if they want to read a write operation that is
 * returned in a history entry.  The history entry will contain the encoded
 * operation, and these functions will decode the operation.  The "args" result
 * will contain pointers to the encoded structure, so keep it in memory while
 * you use args! */
u_int32_t* pasis_md_xdr_dec_create_args (u_int32_t *p, 
                                         pasis_md_create_args_t *args);

u_int32_t* pasis_md_xdr_dec_link_args (u_int32_t *p, 
                                       pasis_md_link_args_t *args);

u_int32_t* pasis_md_xdr_dec_unlink_args (u_int32_t *p, 
                                         pasis_md_unlink_args_t *args);

u_int32_t* pasis_md_xdr_dec_setattr_args (u_int32_t *p, 
                                          pasis_md_setattr_args_t *args);

u_int32_t* pasis_md_xdr_dec_rename_args (u_int32_t *p, 
                                         pasis_md_rename_args_t *args);

u_int32_t* pasis_md_xdr_dec_synth_update_args (u_int32_t *p, 
                                               pasis_md_synth_update_args_t *args);


#ifdef __cplusplus
}
#endif

#endif /* __PASIS_MD_XDR_H__ */

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */ 
