/*
 * The PASIS Query/Update Project
 * Copyright (c) 2007, Carnegie Mellon University.
 * All rights reserved.
 * http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
 *
 * This software is being provided by the copyright holders under the
 * following license. By obtaining, using and/or copying this software,
 * you agree that you have read, understood, and will comply with the
 * following terms and conditions:
 *
 * Permission to reproduce, use, and prepare derivative works of this
 * software is granted provided the copyright and "No Warranty" statements
 * are included with all reproductions and derivative works and associated
 * documentation. This software may also be redistributed without charge
 * provided that the copyright and "No Warranty" statements are included
 * in all redistributions.
 *
 * NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
 * CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
 * EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
 * TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
 * OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
 * MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
 * TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
 * COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
 * OR DOCUMENTATION.
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>

#include "pasis_config.h"
#include "pasis_lists.h"
#include "pasis_timer.h"
#include "pasis_types.h"

#include "pasis_md_auth.h"
#include "pasis_md_prot.h"


#define NUM_HISTORY_ENTRIES 7


p_uint32 node_id;
p_uint32 not_using_backend;


int 
main (int argc, char *argv[])
{
    p_uint32         local_node_id = 5, i;
    pasis_md_hts_t   histories[NUM_HISTORY_ENTRIES];
    pasis_md_hts_t  *histories_pointers[NUM_HISTORY_ENTRIES];
    pasis_md_auth_t  authenticators[PASIS_MAX_NUM_SERVERS];
    int              fd;
    st_netfd_t       rand_fd;

    if ((fd = open ("/dev/urandom", O_RDONLY)) < 0) {
        return PASIS_ERROR;
    }
    rand_fd = st_netfd_open (fd);

    Pasis_TimeInit();
    pasis_md_auth_init();
    pasis_md_auth_stats_init();

    for (i = 0; i < NUM_HISTORY_ENTRIES; ++i) {
#if 0 // for testability
        memset(&histories[i], 0, sizeof(pasis_md_hts_t));
#else

#if USE_OBJECT_HASHES > 0
        st_read_fully (rand_fd, histories[i].object_hash,
                       PASIS_HASH_SIZE, -1);
#endif
        st_read_fully (rand_fd, histories[i].op_hash,
                       PASIS_HASH_SIZE, -1);
        st_read_fully (rand_fd, histories[i].matrix_hash,
                       PASIS_HASH_SIZE, -1);
#endif

        histories[i].time = i * i;

        histories_pointers[i] = &histories[i];
    }

    pasis_md_auth_generate (local_node_id, NUM_HISTORY_ENTRIES,
                            histories, PASIS_MAX_NUM_SERVERS, authenticators);

    for (i = 0; i < PASIS_MAX_NUM_SERVERS; ++i) {
        pasis_error_t error;
        error = pasis_md_auth_verify (i, local_node_id, NUM_HISTORY_ENTRIES,
                                      histories_pointers, &authenticators[i]);
        if (error) {
            fprintf(stderr, "pasis_md_auth_verify returned error\n");
            exit(-1);
        }
    }

    printf("Verification succeeded!\n");

    return 0;
}

/*
 * Local variables:
 *  c-basic-offset: 4
 *  indent-tabs-mode: nil
 * End:
 */
