#!/usr/bin/perl

# The PASIS Query/Update Project
# Copyright (c) 2007, Carnegie Mellon University.
# All rights reserved.
# http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
#
# This software is being provided by the copyright holders under the
# following license. By obtaining, using and/or copying this software,
# you agree that you have read, understood, and will comply with the
# following terms and conditions:
#
# Permission to reproduce, use, and prepare derivative works of this
# software is granted provided the copyright and "No Warranty" statements
# are included with all reproductions and derivative works and associated
# documentation. This software may also be redistributed without charge
# provided that the copyright and "No Warranty" statements are included
# in all redistributions.
#
# NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
# CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
# EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
# TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
# OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
# MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
# TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
# COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
# OR DOCUMENTATION.

$SOURCE_DIR="/h/mabdelm/pasis-qu-release/src";
$RESULTS_DIR="/h/mabdelm/pasis-qu-release/experiments/throughput-scalability";

# Temporary location for each experiment's configuration files. No need to change this.
$BENCH_CONFIG="$RESULTS_DIR/pasisio_md_bench.config";
$METADATA_CONFIG="$RESULTS_DIR/pasis_md_fe.cfg";
$CONTROLLER_CONFIG="$RESULTS_DIR/cw_controller.config";


# Number of runs per experiment
$NUM_RUNS=1;

# We iterate over these possible values of "b"
#@possible_b = (0,1);
@possible_b = (1);

# Iterate over these operation types
# 0=read-hist, 1=small-update, 2=query, 3=update
@possible_operations = (1);

# We iterate over these possible # of clients
@possible_num_clients = (4);

# We iterate over these possible # of outstanding requests
#@possible_num_outstanding = (1,2);
@possible_num_outstanding = (2);


# Rack name.  This is prefixed to the machine names listed in all_clients and
# all_servers.  It's useful in case they all start with a common name prefix.  
# If not, then you can leave it to "".
$rack = "ss";

# A list of all the machines to be used as clients
#@all_clients = (2,3,4,5,6,7,8,9,10,12,13,15,16,19,20,21,22,23,24,25,28,30,32,34,35,36,37,38,39,40,41,42,43,44,45,46);
@all_clients = (9,10,11,12,14,15,16,17);

# A list of all the machines to be used as servers.
#@all_servers = (52,53,54,55,56,57,58,59,60,61,62,63,64,65,66,67,68,69,70,71,72,73,74,75,77,78);
@all_servers = (1,2,3,4,6,8,18,19,21,22,23);

$total_num_experiments = $NUM_RUNS * @possible_b * @possible_num_outstanding *
    @possible_num_clients * @possible_operations;


$quorum_config = "\n
QUORUM=threshold_comp_qc_b
DELTA=0\n\n";

$query_config = "\n
OPERATION_NAME=query
OPERATION_TYPE=query
NUM_OBJECTS=1
ARG_SIZE=0
RES_SIZE=4096
NUM_READ_CHUNKS=1
CLIENT_COMPUTE_TIME=0
SERVER_COMPUTE_TIME=0
RELATIVE_FREQUENCY=1
CHUNK_SIZE=4096\n\n";


$update_config = "\n
OPERATION_NAME=update
OPERATION_TYPE=update
NUM_OBJECTS=1
ARG_SIZE=4096
RES_SIZE=0
NUM_WRITTEN_CHUNKS=1
CLIENT_COMPUTE_TIME=0
SERVER_COMPUTE_TIME=0
RELATIVE_FREQUENCY=1
CHUNK_SIZE=4096\n\n";


$read_hist_config = "\n
OPERATION_NAME=read-history
OPERATION_TYPE=query
NUM_OBJECTS=1
ARG_SIZE=0
RES_SIZE=0
NUM_READ_CHUNKS=0
CLIENT_COMPUTE_TIME=0
SERVER_COMPUTE_TIME=0
RELATIVE_FREQUENCY=1
CHUNK_SIZE=4\n\n";

$small_update_config = "\n
OPERATION_NAME=small-update
OPERATION_TYPE=update
NUM_OBJECTS=1
ARG_SIZE=4
RES_SIZE=0
NUM_WRITTEN_CHUNKS=1
CLIENT_COMPUTE_TIME=0
SERVER_COMPUTE_TIME=0
RELATIVE_FREQUENCY=1
CHUNK_SIZE=4\n\n";


$benchmark_config="\n
RANDOMLY_SELECT_FID=0\n
FID_START=0xdeadbeef\n
READ_RETRIES=4\n
WARMUP_TIME=5\n
CALIB_TIME=5\n
RUN_TIME=10\n
SLOWDOWN_TIME=10\n";

# To run without _calculating_ authenticators (i.e., use memset(0) instead of any
# hash()), add -A after pasis_md_fe.
# To run without _transferring_ authenticators on the network, add a -T after
# pasis_md_fe.  This implies -A.
# To force a read-history before every update, add a "-H" in the CLIENT_COMMAND, 
# before the "-c".
# To force random quorums (instead of preferred quorums), add a '-R' in the
# CLIENT_COMMAND, before the "-c".
$controller_config="\n
RESULTS_DIR=$RESULTS_DIR\n
MD_FE_CMD=$SOURCE_DIR/s4/md_fe/pasis_md_fe\n
MD_FS_CMD=\n
MOUNT_POINT=\n
CLIENT_FORMAT_COMMAND=$SOURCE_DIR/benchmarks/pasisio_md_bench -c CLIENTID -f $BENCH_CONFIG\n
CLIENT_COMMAND=$SOURCE_DIR/benchmarks/pasisio_md_bench -c CLIENTID $BENCH_CONFIG\n";


# Copy this script to the results directory
create_dirpath ($RESULTS_DIR);
system ("cp $0 $RESULTS_DIR");


# Start daemons on clients and servers.  The daemons listen for commands to execute.
start_daemons(@all_clients);
start_daemons(@all_servers);


$exp_num = 0;

# Iterate for all "B" values
foreach my $b (@possible_b) {
    my $N = 5 * $b + 1;
    my $server_list = get_server_list ($N);
    my $bench_config = $server_list;
    $bench_config .= "B=$b\n";
    $bench_config .= "T=$b\n";
    $bench_config .= $quorum_config;
    $bench_config .= "NUM_OPERATION_TYPES=1\n";

    my $md_fe_config = $server_list;
    $md_fe_config .= "B=$b\n";
    $md_fe_config .= "T=$b\n";
    $md_fe_config .= $quorum_config;

    # Now, create the metadata front-end config file
    open(MD_CONFIG_FILE, ">$METADATA_CONFIG");
    print MD_CONFIG_FILE $md_fe_config;


    # Iterate over the operation types
    foreach my $operation (@possible_operations) {
        my $total_bench_config, $op_name;
        $total_bench_config = $bench_config;

        # First, get the appropriate operation definition for the benchmark config file
        if ($operation == 0) {
            $total_bench_config .= $read_hist_config;
            $op_name = "read-hist";
        } elsif ($operation == 1) {
            $total_bench_config .= $small_update_config;
            $op_name = "small-update";
        } elsif ($operation == 2) {
            $total_bench_config .= $query_config;
            $op_name = "query";
        } elsif ($operation == 3) {
            $total_bench_config .= $update_config;
            $op_name = "update";
        }

        # Iterate over # of clients
        foreach my $num_clients (@possible_num_clients) {
            my $total_controller_config;
            $total_controller_config = $controller_config;
            $total_controller_config .= get_client_list($num_clients);

            # Now, create the controller configuration file
            open(CONTROLLER_CONFIG_FILE, ">$CONTROLLER_CONFIG");
            print CONTROLLER_CONFIG_FILE $total_controller_config;

            # Iterate over # of outstanding requests
            foreach my $num_out (@possible_num_outstanding) {
                # At a minimum, we need the number of objects per client to be at 
                # least NUM_OUTSTANDING, to avoid intra-client concurrency. 
                # my $final_total_bench_config = $total_bench_config . "NUM_OBJECTS=$num_out\n";
                my $final_total_bench_config = $total_bench_config . "NUM_OBJECTS=128\n";
                $final_total_bench_config .= "NUM_OUTSTANDING=$num_out\n";
                $final_total_bench_config .= $benchmark_config;

                # Now, create the benchmark configuration file
                open(BENCH_CONFIG_FILE, ">$BENCH_CONFIG");
                print BENCH_CONFIG_FILE $final_total_bench_config;

                # Do $NUM_RUNS runs of each experiment
                for (my $k = 1; $k <= $NUM_RUNS; $k++) {
                    my $exp_name = "$op_name-b_$b-num_clients_$num_clients-num_outstanding_$num_out-run$k";
                    $exp_num++;

                    print ("************************************************************\n");
                    print ("Performing experiment: $exp_name ($exp_num of $total_num_experiments)\n");
                    print ("************************************************************\n");
                    
                    # Execute the experiment
                    system("$SOURCE_DIR/tools/scripts/cw_controller.pl -n $exp_name -m $METADATA_CONFIG -c $CONTROLLER_CONFIG");
                }
            }
        }
    }
}


kill_daemons(@all_clients);
kill_daemons(@all_servers);

exit;


sub get_client_list {
    my ($N) = @_;
    my $output, $client_num;

    $output = "CLIENT_LIST=";

    if ($N > @all_clients) {
        my $num_clients = @all_clients;
        print "Error: not enough clients (N=$N, num_clients = $num_clients).\n";
        exit;
    }

    $client_num = @all_clients[$0];
    $output .= "$rack$client_num";
    for (my $k = 1; $k < $N; $k++) {
        $client_num = @all_clients[$k];
        $output .= ",$rack$client_num";
    }

    return $output . "\n";
}

sub get_server_list {
    my ($N) = @_;
    my ($output);

    $output = "NUM_SERVERS=$N\n";

    if ($N > @all_servers) {
        my $num_servers = @all_servers;
        print "Error: not enough servers (N=$N, num_servers = $num_servers).\n";
        exit;
    }

    for (my $k = 0; $k < $N; $k++) {
        $server_num = @all_servers[$k];
        $server_line = "$rack$server_num:/1"; # ss3:/1
        $output .= "$server_line\n";
    }

    $output .= "\n";

    return $output;
}


sub start_daemons {
    my @machine_list = @_;
    for (my $i = 0; $i < @machine_list; $i++) {
        system("ssh -f -o ConnectTimeout=1 -SetupTimeOut=1 $rack@machine_list[$i] \"(nohup $SOURCE_DIR/tools/scripts/cw_daemon.pl &)\"");
    }
    # Give the daemons time to start up
    sleep(2);
}


sub kill_daemons {
    my @machine_list = @_;
    for (my $i = 0; $i < @machine_list; $i++) {
        system("ssh -o ConnectTimeout=1 -SetupTimeOut=1 $rack@machine_list[$i] \"killall -9 cw_daemon.pl\"");
    }
}


sub create_dirpath {
    my ($path) = @_;
    my @paths = split (/\//, $path);
    my $full_path="";

    foreach $path (@paths) {
        if ($path) {
            $full_path .= "/" . $path;

            unless (-d $full_path) {
                my $ret = mkdir $full_path, 0777;
                if ($ret == 0) {
                    print "Creating results dir ($full_path) failed ($!)";
                    return -1;
                }
            }
        }
    }

    return 0;
}
