#!/usr/bin/perl

# The PASIS Query/Update Project
# Copyright (c) 2007, Carnegie Mellon University.
# All rights reserved.
# http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
#
# This software is being provided by the copyright holders under the
# following license. By obtaining, using and/or copying this software,
# you agree that you have read, understood, and will comply with the
# following terms and conditions:
#
# Permission to reproduce, use, and prepare derivative works of this
# software is granted provided the copyright and "No Warranty" statements
# are included with all reproductions and derivative works and associated
# documentation. This software may also be redistributed without charge
# provided that the copyright and "No Warranty" statements are included
# in all redistributions.
#
# NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
# CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
# EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
# TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
# OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
# MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
# TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
# COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
# OR DOCUMENTATION.

use IO::Socket;
use FileHandle;

use Getopt::Std;

sub logmsg {print "$0: @_ at ", scalar localtime, "\n";}

getopts ('p:n:m:c:w:h:r:');

my $port = 45678;

my $config_fh = FileHandle->new ();

unless (-e $opt_m and -e $opt_c and $opt_n) {
    usage ();
    exit 0;
}

if ($opt_h) {
    usage ();
    exit 0;
}

my $client_processes=1;
if ($opt_p) {
    $client_processes = $opt_p;
}

open (MDFILE, $opt_m) || die "File not found: $opt_m\n";
while (<MDFILE>) {
    chomp;
    next if (/^\s*\#/);
    push @md_cfg, $_;
    
    if (/^(.*?):\/(\d+)/) {
        push @servers, $1;
        push @parts, $2;
    }
}

$exp_name = $opt_n;

open ($config_fh, $opt_c) || die "Config file not found: $!\n";

$results_dir = get_line ($config_fh, "RESULTS_DIR");
$md_fe_cmd = get_line ($config_fh, "MD_FE_CMD");
$md_fs_cmd = get_line ($config_fh, "MD_FS_CMD");
$mount_point = get_line ($config_fh, "MOUNT_POINT");
$client_format_cmd = get_line ($config_fh, "CLIENT_FORMAT_COMMAND");
$client_cmd = get_line ($config_fh, "CLIENT_COMMAND");
$client_list = get_line ($config_fh, "CLIENT_LIST");

close $config_fh;

$results_dir =~ s/^\s*//o;
$results_dir =~ s/\s*$//o;

$md_fe_cmd =~ s/^\s*//o;
$md_fe_cmd =~ s/\s*$//o;

$md_fs_cmd =~ s/^\s*//o;
$md_fs_cmd =~ s/\s*$//o;

$client_format_cmd =~ s/^\s*//o;
$client_format_cmd =~ s/\s*$//o;

$client_cmd =~ s/^\s*//o;
$client_cmd =~ s/\s*$//o;

unless ($results_dir =~ /^\//) {
    die "RESULTS_DIR must be an absolute pathname ($results_dir)\n";
}

$results_dir .= "/" . $exp_name . "/";

$results_dir =~ s/\s+/_/go;
$results_dir =~ s%/+%/%go;

$client_list =~ s/\s//go;
@clients = split (/,/, $client_list);

print "RESULTS_DIR: ", $results_dir, "\n";
print "MD_FE: ", $md_fe_cmd, "\n";
print "MD_FS: ", $md_fs_cmd, "\n";
print "MOUNT_POINT: ", $mount_point, "\n";
print "CLIENT_FORMAT_COMMAND: ", $client_format_cmd, "\n" ;
print "CLIENT_COMMAND: ", $client_cmd, "\n" ;

print "Serverlist: @servers\n";
print "Clientlist: @clients\n";



# Copy the configuration files to the results directory
create_dirpath ($results_dir);
system ("cp $opt_m $results_dir");
system ("cp $opt_c $results_dir");
@possible_config_files = split (/ /, $client_cmd);
foreach $possible_config_file (@possible_config_files) {
    if ($possible_config_file =~ /config/ ||
        $possible_config_file =~ /cfg/) {
        system ("cp $possible_config_file $results_dir");
    }
}


# Start each server
$i=0;
foreach $server_name (@servers)  {

    logmsg "Connecting to server -- $server_name for EXPERIMENT";

    $server_fh = create_conn ($server_name, $port);
    $server_fh->print ("EXPERIMENT\n");
    $server_fh->print ($results_dir, "\n");

    $server_fh->print (scalar (@md_cfg), "\n");
    foreach $md_cfg_line (@md_cfg) {
        $server_fh->print ($md_cfg_line, "\n");
    }

    $server_fh->print ("SERVER\n");
    # Node ID
    $server_fh->print ($i, "\n");
    $server_fh->print ($md_fe_cmd, "\n");

    push @server_fhs, $server_fh;
    $i++;
}

sleep (2);


$i=0;
$n=@clients;
foreach $client_name (@clients) {

    logmsg "Connecting to client -- $client_name for EXPERIMENT";

    $client_fh = create_conn ($client_name, $port);

    $client_fh->print ("EXPERIMENT\n");
    $client_fh->print ($results_dir, "\n");

    $client_fh->print (scalar (@md_cfg), "\n");
    foreach $md_cfg_line (@md_cfg) {
        $client_fh->print ($md_cfg_line, "\n");
    }

    $client_fh->print ("CLIENT\n");

    # Node ID
    
    # For contention experiments, the -w option specifies the number 
    # of clients sharing the same client id.
    if ($opt_w && $i < $opt_w) {
        $client_fh->print (0, "\n");
    } else {
        $client_fh->print ($i, "\n");
    }

    $client_fh->print ($n, "\n");
    $client_fh->print ($client_processes, "\n");
    $client_fh->print ($md_fs_cmd, "\n");
    $client_fh->print ($mount_point, "\n");
    $client_fh->print ($client_format_cmd, "\n");

    # For read-write contention experiments, reader clients get a different
    # command (with a "reader" config file, isntead of the "writer" config
    # file).   The writer is client ID 1; the rest are readers.
    if ($opt_r && $i >= 1) {
        $client_fh->print ($opt_r, "\n");
    } else {
        $client_fh->print ($client_cmd, "\n");
    }

#    $client_fh->print ($i * 2, "\n");
#    $client_fh->print ($n * 2, "\n");

    push @client_fhs, $client_fh;
    $i++;
}


# Wait for each client to finish initializating
$i = 0;
foreach $client_fh (@client_fhs) {
    logmsg "Waiting for client $i to finish initializing";

    $status = $client_fh->getline ();
    chomp ($status);

    $error = 1 unless $status =~ /INITIALIZATION_DONE/;

    $i++;
}



# Now, start each client
foreach $client_fh (@client_fhs) {

    logmsg "Issuing client start";

    $client_fh->print ("START\n");
}

$error = 0;

# Wait for each client to stop
$i = 0;
foreach $client_fh (@client_fhs) {
    logmsg "Waiting for client";

    $status = $client_fh->getline ();
    chomp ($status);

    $error = 1 unless $status =~ /DONE/;

    logmsg "Client: ".$clients[$i]." returned status=".$status;

    #don't die because we want to get the error conditions for other
    #servers and clients if they are different
    #die "Client Error: ",$clients[$i],"\n" unless $status =~ /DONE/;
    
    $client_fh->close ();

    $i++;
}



# Stop each server.  Do this in two phases: send STOP commands in parallal,
# as we want to parallelize the slow task of S4 writing its cache to disk.
# Then, wait for the DONE return line.
foreach $server_fh (@server_fhs) {
    logmsg "Stopping server";
    $server_fh->print ("STOP\n");
}

$i = 0;
foreach $server_fh (@server_fhs) {
    $status = $server_fh->getline ();
    chomp ($status);

    $error = 1 unless $status =~ /DONE/;

    logmsg "Server: ".$servers[$i]." returned status=".$status;

    #don't die because we want to get the error conditions for other
    #servers if they are different
    # die "Server Error: ",$servers[$i],"\n" unless $status =~ /DONE/;

    $server_fh->close ();

    $i++;
}


exit $error;


sub get_line {
    my ($fh, $name) = @_;
    my ($output);

    $output = "";

    while (<$fh>) {
        next if (/^\s*\#/);
        next if (/^\s*$/);

        chomp;
        
        /(.*)=(.*)/;
        if ($1 ne $name) {
            die "Parse error: found $1, expecting $name\n";
        }
        
        $output = $2;
        return $output;
    }

    die "Parse error: end of file reached unexpectedly\n";
}

sub create_conn2 {
    my ($client) = @_;
    
    my $fh = FileHandle->new ();

    open ($fh, ">$exp_name.$client") || die "Can't open file: $!\n";
    
    return $fh;
}


sub create_conn {

    my ($remote_host, $port) = @_;

    my $sock = new IO::Socket::INET (
                                     PeerAddr => $remote_host,
                                     PeerPort => $port,
                                     Proto => 'tcp',
                                     );


    die "Could not create socket to $remote_host: $! ($@)\n" unless $sock;

    $sock->autoflush(1);

    return $sock;
}


sub create_dirpath {
    my ($path) = @_;
    my @paths = split (/\//, $path);
    my $full_path="";

    foreach $path (@paths) {
        if ($path) {
            $full_path .= "/" . $path;

            unless (-d $full_path) {
                my $ret = mkdir $full_path, 0777;
                if ($ret == 0) {
                    logmsg "Creating results dir ($full_path) failed ($!)";
                    return -1;
                }
            }
        }
    }

    return 0;
}


sub usage {
    
    print "$0 -n expt_name -m md.cfg -c expt.cfg\n";
    print "  -n expt_name    Experiment name, sub dir is created\n";
    print "  -m md.cfg       Metadata config file\n";
    print "  -c expt.cfg     Controller config file\n";
    print "  -p              Number of sub processes to spawn (default=1)\n";
    print "  For contention experiments\n";
    print "  -w <num>        # of clients using same object\n";
    print "  For read-write contention experiments\n";
    print "  -r <reader cmd> Client command to use for readers\n";
    print "\n";
}
