#!/usr/bin/perl

# The PASIS Query/Update Project
# Copyright (c) 2007, Carnegie Mellon University.
# All rights reserved.
# http://www.pdl.cmu.edu/  (Parallel Data Lab at Carnegie Mellon)
#
# This software is being provided by the copyright holders under the
# following license. By obtaining, using and/or copying this software,
# you agree that you have read, understood, and will comply with the
# following terms and conditions:
#
# Permission to reproduce, use, and prepare derivative works of this
# software is granted provided the copyright and "No Warranty" statements
# are included with all reproductions and derivative works and associated
# documentation. This software may also be redistributed without charge
# provided that the copyright and "No Warranty" statements are included
# in all redistributions.
#
# NO WARRANTY. THIS SOFTWARE IS FURNISHED ON AN "AS IS" BASIS.
# CARNEGIE MELLON UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER
# EXPRESSED OR IMPLIED AS TO THE MATTER INCLUDING, BUT NOT LIMITED
# TO: WARRANTY OF FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY
# OF RESULTS OR RESULTS OBTAINED FROM USE OF THIS SOFTWARE. CARNEGIE
# MELLON UNIVERSITY DOES NOT MAKE ANY WARRANTY OF ANY KIND WITH RESPECT
# TO FREEDOM FROM PATENT, TRADEMARK, OR COPYRIGHT INFRINGEMENT.
# COPYRIGHT HOLDERS WILL BEAR NO LIABILITY FOR ANY USE OF THIS SOFTWARE
# OR DOCUMENTATION.


open(RESULTS_FILE, ">results");

$last_exp = "";
$exp_total_tps = 0;
$exp_total_mean_rt = 0;
$exp_num = 0;

while (<*run*>) {
    $dir = $_;

    print "Examining directory: ", $dir, "\n";
    
    ($dir_tps, $dir_mean_rt) = summarize_dir ($dir);

    # Print the average if we're at the finish of a set of runs
    $dir =~ /([\w-=]+)-run[\d]+/;
    if ($1 eq $last_exp) {
        $exp_total_tps += $dir_tps;
        $exp_total_mean_rt += $dir_mean_rt;
        $exp_num++;
    } else {
        if ($exp_num == 0) {
            # Ignore first time
        } else { # if ($exp_num != 1) {
            $avg_tps = $exp_total_tps / $exp_num;
            $avg_mean_rt = $exp_total_mean_rt / $exp_num;
            print RESULTS_FILE "$last_exp average mean response time: $avg_mean_rt ms\n";
            print RESULTS_FILE "$last_exp average throughput: $avg_tps req/s\n";
        }
        $exp_total_tps = $dir_tps;
        $exp_total_mean_rt = $dir_mean_rt;
        $exp_num = 1;
    }
    $last_exp = $1;

    # print RESULTS_FILE "$dir: $dir_tps\n";
}

$avg_tps = $exp_total_tps / $exp_num;
$avg_mean_rt = $exp_total_mean_rt / $exp_num;
print RESULTS_FILE "$last_exp average mean response time: $avg_mean_rt ms\n";
print RESULTS_FILE "$last_exp average throughput: $avg_tps req/s\n";


# Sort files by changing "1" to "01", etc...
`perl -pi -e 's/b_0/b_00/' results`;
`perl -pi -e 's/b_1/b_01/' results`;
`perl -pi -e 's/b_2/b_02/' results`;
`perl -pi -e 's/b_3/b_03/' results`;
`perl -pi -e 's/b_4/b_04/' results`;
`perl -pi -e 's/b_5/b_05/' results`;
`perl -pi -e 's/b_6/b_06/' results`;
`perl -pi -e 's/b_7/b_07/' results`;
`perl -pi -e 's/b_8/b_08/' results`;
`perl -pi -e 's/b_9/b_09/' results`;
`perl -pi -e 's/_1-/_01-/' results`;
`perl -pi -e 's/_2-/_02-/' results`;
`perl -pi -e 's/_3-/_03-/' results`;
`perl -pi -e 's/_4-/_04-/' results`;
`perl -pi -e 's/_5-/_05-/' results`;
`perl -pi -e 's/_6-/_06-/' results`;
`perl -pi -e 's/_7-/_07-/' results`;
`perl -pi -e 's/_8-/_08-/' results`;
`perl -pi -e 's/_9-/_09-/' results`;
`perl -pi -e 's/_1 /_01 /' results`;
`perl -pi -e 's/_2 /_02 /' results`;
`perl -pi -e 's/_3 /_03 /' results`;
`perl -pi -e 's/_4 /_04 /' results`;
`perl -pi -e 's/_5 /_05 /' results`;
`perl -pi -e 's/_6 /_06 /' results`;
`perl -pi -e 's/_7 /_07 /' results`;
`perl -pi -e 's/_8 /_08 /' results`;
`perl -pi -e 's/_9 /_09 /' results`;
`sort results > results2`;
`mv results2 results`;




sub summarize_dir {
    my ($dir) = @_;
    my $total_tps = 0;
    my $total_response_time = 0;
    my $count = 0;

    while (<$dir/*_0_cmd.out>) {
        my $filename = $_;
        $tps = `grep "Total Throughput" $filename`;
        $tps =~ /([.\d]+) req\/s/;
        $tps = $1;
        $total_tps += $tps;

        $rt = `grep "Measurement:" $filename`;
        $rt =~ /mean=([.\d]+)ms/;
        $rt = $1;
        $total_response_time += $rt;

        ++$count;
    }


    #if (true) { # print info for each run
    #    print RESULTS_FILE "$dir throughput: $total_tps req/s\n";
    #}
                                 
    return $total_tps, $total_response_time / $count;
}
